# Interoperable data formats

The diversity of raw data file formats generated by different sonar systems is a major obstacle toward integrative analysis of ocean sonar data at large scales. Echopype addresses this problem by providing tools for converting and standardizing data from manufacturer-specific formats into the **[netCDF](https://www.unidata.ucar.edu/software/netcdf/) data model**, so that data from different instruments are interoperable. NetCDF is the [current defacto standard](https://clouds.eos.ubc.ca/~phil/courses/parallel_python/02_xarray_zarr.html) in climate research and is supported by many powerful Python packages for efficient computation, of which echopype takes advantage extensively.

See the following sections:

- [**Adaptation of SONAR-netCDF4 convention**](data-format:sonarnetcdf4-adaptation) details echopype's adaptation of the [ICES SONAR-netCDF4 convention version 1](https://ices-library.figshare.com/articles/report/The_SONAR-netCDF4_convention_for_sonar_data_Version_1_0/18624056) to create standardized data that fully leverage the power of label-aware manipulation and enhance coherence of data representation for scientific echosounders.
- [**Raw converted data**](data-format:raw-data) describes the standardized [`EchoData` object](data-format:echodata-object) and provides instrument-specific examples on raw data unpacked and organized ("converted" via [`open_raw`](echopype.open_raw)) into `EchoData` objects.
- [**Changes in recent versions**](data-format-changes.md) describes significant changes in data structures in recent major versions of echopype.
